@@ -19,13 +19,13 @@
 #include <algorithm>
 #include <utility>
 
+#include "absl/log/log.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/optional.h"
 #include "absl/types/span.h"
 #include "upb/base/string_view.h"
 #include "upb/message/map.h"
 
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 
 namespace grpc_core {
@@ -50,16 +50,14 @@ std::unique_ptr<CelAuthorizationEngine>
 CelAuthorizationEngine::CreateCelAuthorizationEngine(
     const std::vector<envoy_config_rbac_v3_RBAC*>& rbac_policies) {
   if (rbac_policies.empty() || rbac_policies.size() > 2) {
-    gpr_log(GPR_ERROR,
-            "Invalid rbac policies vector. Must contain either one or two rbac "
-            "policies.");
+    LOG(ERROR) << "Invalid rbac policies vector. Must contain either one or "
+                  "two rbac policies.";
     return nullptr;
   } else if (rbac_policies.size() == 2 &&
              (envoy_config_rbac_v3_RBAC_action(rbac_policies[0]) != kDeny ||
               envoy_config_rbac_v3_RBAC_action(rbac_policies[1]) != kAllow)) {
-    gpr_log(GPR_ERROR,
-            "Invalid rbac policies vector. Must contain one deny \
-                         policy and one allow policy, in that order.");
+    LOG(ERROR) << "Invalid rbac policies vector. Must contain one deny policy "
+                  "and one allow policy, in that order.";
     return nullptr;
   } else {
     return std::make_unique<CelAuthorizationEngine>(rbac_policies);
@@ -175,10 +173,9 @@ std::unique_ptr<mock_cel::Activation> CelAuthorizationEngine::CreateActivation(
             mock_cel::CelValue::CreateStringView(cert_server_name));
       }
     } else {
-      gpr_log(GPR_ERROR,
-              "Error: Authorization engine does not support evaluating "
-              "attribute %s.",
-              elem.c_str());
+      LOG(ERROR) << "Error: Authorization engine does not support evaluating "
+                    "attribute "
+                 << elem;
     }
   }
   return activation;
