@@ -24,6 +24,7 @@
 #include <vector>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
@@ -34,7 +35,6 @@
 #include <grpc/grpc_security.h>
 #include <grpc/impl/connectivity_state.h>
 #include <grpc/support/json.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 
 #include "src/core/lib/channel/channel_args.h"
@@ -49,10 +49,6 @@
 #include "src/core/lib/gprpp/unique_type_name.h"
 #include "src/core/lib/gprpp/work_serializer.h"
 #include "src/core/lib/iomgr/pollset_set.h"
-#include "src/core/lib/json/json.h"
-#include "src/core/lib/json/json_args.h"
-#include "src/core/lib/json/json_object_loader.h"
-#include "src/core/lib/json/json_writer.h"
 #include "src/core/load_balancing/address_filtering.h"
 #include "src/core/load_balancing/delegating_helper.h"
 #include "src/core/load_balancing/lb_policy.h"
@@ -61,14 +57,16 @@
 #include "src/core/load_balancing/outlier_detection/outlier_detection.h"
 #include "src/core/load_balancing/xds/xds_channel_args.h"
 #include "src/core/resolver/xds/xds_dependency_manager.h"
+#include "src/core/util/json/json.h"
+#include "src/core/util/json/json_args.h"
+#include "src/core/util/json/json_object_loader.h"
+#include "src/core/util/json/json_writer.h"
 #include "src/core/xds/grpc/xds_cluster.h"
 #include "src/core/xds/grpc/xds_common_types.h"
 #include "src/core/xds/grpc/xds_health_status.h"
 
 namespace grpc_core {
 
-TraceFlag grpc_cds_lb_trace(false, "cds_lb");
-
 namespace {
 
 // TODO(roth): Remove this after the 1.63 release.
@@ -185,20 +183,20 @@ class CdsLb final : public LoadBalancingPolicy {
 //
 
 CdsLb::CdsLb(Args args) : LoadBalancingPolicy(std::move(args)) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] created", this);
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this << "] created";
   }
 }
 
 CdsLb::~CdsLb() {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] destroying cds LB policy", this);
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this << "] destroying cds LB policy";
   }
 }
 
 void CdsLb::ShutdownLocked() {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] shutting down", this);
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this << "] shutting down";
   }
   shutting_down_ = true;
   ResetState();
@@ -282,9 +280,10 @@ class PriorityEndpointIterator final : public EndpointAddressesIterator {
 absl::Status CdsLb::UpdateLocked(UpdateArgs args) {
   // Get new config.
   auto new_config = args.config.TakeAsSubclass<CdsLbConfig>();
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] received update: cluster=%s is_dynamic=%d",
-            this, new_config->cluster().c_str(), new_config->is_dynamic());
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this
+              << "] received update: cluster=" << new_config->cluster()
+              << " is_dynamic=" << new_config->is_dynamic();
   }
   CHECK(new_config != nullptr);
   // Cluster name should never change, because we should use a different
@@ -296,10 +295,10 @@ absl::Status CdsLb::UpdateLocked(UpdateArgs args) {
   }
   // Start dynamic subscription if needed.
   if (new_config->is_dynamic() && subscription_ == nullptr) {
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-      gpr_log(GPR_INFO,
-              "[cdslb %p] obtaining dynamic subscription for cluster %s", this,
-              cluster_name_.c_str());
+    if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+      LOG(INFO) << "[cdslb " << this
+                << "] obtaining dynamic subscription for cluster "
+                << cluster_name_;
     }
     auto* dependency_mgr = args.args.GetObject<XdsDependencyManager>();
     if (dependency_mgr == nullptr) {
@@ -327,11 +326,10 @@ absl::Status CdsLb::UpdateLocked(UpdateArgs args) {
       // If we are already subscribed, it's possible that we just
       // recently subscribed but another update came through before we
       // got the new cluster, in which case it will still be missing.
-      if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-        gpr_log(GPR_INFO,
-                "[cdslb %p] xDS config has no entry for dynamic cluster %s, "
-                "waiting for subsequent update",
-                this, cluster_name_.c_str());
+      if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+        LOG(INFO) << "[cdslb " << this
+                  << "] xDS config has no entry for dynamic cluster "
+                  << cluster_name_ << ", waiting for subsequent update";
       }
       // Stay in CONNECTING until we get an update that has the cluster.
       return absl::OkStatus();
@@ -454,10 +452,10 @@ absl::Status CdsLb::UpdateLocked(UpdateArgs args) {
     }
     grpc_pollset_set_add_pollset_set(child_policy_->interested_parties(),
                                      interested_parties());
-    if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-      gpr_log(GPR_INFO, "[cdslb %p] created child policy %s (%p)", this,
-              std::string((*child_config)->name()).c_str(),
-              child_policy_.get());
+    if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+      LOG(INFO) << "[cdslb " << this << "] created child policy "
+                << (*child_config)->name() << " (" << child_policy_.get()
+                << ")";
     }
   }
   // Update child policy.
@@ -664,9 +662,9 @@ Json CdsLb::CreateChildPolicyConfigForLeafCluster(
       {"outlier_detection_experimental",
        Json::FromObject(std::move(outlier_detection_config))},
   })});
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] generated config for child policy: %s", this,
-            JsonDump(outlier_detection_policy, /*indent=*/1).c_str());
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this << "] generated config for child policy: "
+              << JsonDump(outlier_detection_policy, /*indent=*/1);
   }
   return outlier_detection_policy;
 }
@@ -697,9 +695,9 @@ Json CdsLb::CreateChildPolicyConfigForAggregateCluster(
            {"priorities", Json::FromArray(std::move(priority_priorities))},
        })},
   })});
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] generated config for child policy: %s", this,
-            JsonDump(json, /*indent=*/1).c_str());
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this << "] generated config for child policy: "
+              << JsonDump(json, /*indent=*/1);
   }
   return json;
 }
@@ -716,9 +714,9 @@ void CdsLb::ResetState() {
 }
 
 void CdsLb::ReportTransientFailure(absl::Status status) {
-  if (GRPC_TRACE_FLAG_ENABLED(grpc_cds_lb_trace)) {
-    gpr_log(GPR_INFO, "[cdslb %p] reporting TRANSIENT_FAILURE: %s", this,
-            status.ToString().c_str());
+  if (GRPC_TRACE_FLAG_ENABLED(cds_lb)) {
+    LOG(INFO) << "[cdslb " << this
+              << "] reporting TRANSIENT_FAILURE: " << status;
   }
   ResetState();
   channel_control_helper()->UpdateState(
