@@ -26,6 +26,7 @@
 #include "absl/base/thread_annotations.h"
 #include "absl/cleanup/cleanup.h"
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/match.h"
@@ -35,7 +36,6 @@
 
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/impl/channel_arg_names.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 
 #include "src/core/lib/backoff/backoff.h"
@@ -77,16 +77,10 @@ using grpc_event_engine::experimental::EventEngine;
 // TODO(hork): Add a test that checks for proper authority from balancer
 // addresses.
 
-// TODO(hork): replace this with `dns_resolver` when all other resolver
-// implementations are removed.
-TraceFlag grpc_event_engine_client_channel_resolver_trace(
-    false, "event_engine_client_channel_resolver");
-
-#define GRPC_EVENT_ENGINE_RESOLVER_TRACE(format, ...)                    \
-  if (GRPC_TRACE_FLAG_ENABLED(                                           \
-          grpc_event_engine_client_channel_resolver_trace)) {            \
-    gpr_log(GPR_DEBUG, "(event_engine client channel resolver) " format, \
-            __VA_ARGS__);                                                \
+#define GRPC_EVENT_ENGINE_RESOLVER_TRACE(format, ...)                  \
+  if (GRPC_TRACE_FLAG_ENABLED(event_engine_client_channel_resolver)) { \
+    VLOG(2) << "(event_engine client channel resolver) "               \
+            << absl::StrFormat(format, __VA_ARGS__);                   \
   }
 
 // ----------------------------------------------------------------------------
@@ -184,7 +178,7 @@ EventEngineClientChannelDNSResolver::EventEngineClientChannelDNSResolver(
                           .set_jitter(GRPC_DNS_RECONNECT_JITTER)
                           .set_max_backoff(Duration::Milliseconds(
                               GRPC_DNS_RECONNECT_MAX_BACKOFF_SECONDS * 1000)),
-                      &grpc_event_engine_client_channel_resolver_trace),
+                      &event_engine_client_channel_resolver_trace),
       request_service_config_(
           !channel_args()
                .GetBool(GRPC_ARG_SERVICE_CONFIG_DISABLE_RESOLUTION)
@@ -567,7 +561,7 @@ absl::optional<Resolver::Result> EventEngineClientChannelDNSResolver::
 bool EventEngineClientChannelDNSResolverFactory::IsValidUri(
     const URI& uri) const {
   if (absl::StripPrefix(uri.path(), "/").empty()) {
-    gpr_log(GPR_ERROR, "no server name supplied in dns URI");
+    LOG(ERROR) << "no server name supplied in dns URI";
     return false;
   }
   return true;
