@@ -23,6 +23,7 @@
 #include <string>
 #include <vector>
 
+#include "absl/log/log.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/strip.h"
 #include "envoy/config/core/v3/base.upb.h"
@@ -43,12 +44,11 @@
 #include "upb/text/encode.h"
 
 #include <grpc/status.h>
-#include <grpc/support/log.h>
 #include <grpc/support/port_platform.h>
 #include <grpc/support/time.h>
 
-#include "src/core/lib/json/json.h"
-#include "src/core/xds/grpc/upb_utils.h"
+#include "src/core/util/json/json.h"
+#include "src/core/util/upb_utils.h"
 #include "src/core/xds/xds_client/xds_client.h"
 
 // IWYU pragma: no_include "upb/msg_internal.h"
@@ -130,15 +130,14 @@ void PopulateMetadataValue(google_protobuf_Value* value_pb, const Json& value,
 void MaybeLogDiscoveryRequest(
     const XdsApiContext& context,
     const envoy_service_discovery_v3_DiscoveryRequest* request) {
-  if (GRPC_TRACE_FLAG_ENABLED(*context.tracer) &&
-      gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+  if (GRPC_TRACE_FLAG_ENABLED_OBJ(*context.tracer) && ABSL_VLOG_IS_ON(2)) {
     const upb_MessageDef* msg_type =
         envoy_service_discovery_v3_DiscoveryRequest_getmsgdef(context.def_pool);
     char buf[10240];
     upb_TextEncode(reinterpret_cast<const upb_Message*>(request), msg_type,
                    nullptr, 0, buf, sizeof(buf));
-    gpr_log(GPR_DEBUG, "[xds_client %p] constructed ADS request: %s",
-            context.client, buf);
+    VLOG(2) << "[xds_client " << context.client
+            << "] constructed ADS request: " << buf;
   }
 }
 
@@ -262,16 +261,15 @@ namespace {
 void MaybeLogDiscoveryResponse(
     const XdsApiContext& context,
     const envoy_service_discovery_v3_DiscoveryResponse* response) {
-  if (GRPC_TRACE_FLAG_ENABLED(*context.tracer) &&
-      gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+  if (GRPC_TRACE_FLAG_ENABLED_OBJ(*context.tracer) && ABSL_VLOG_IS_ON(2)) {
     const upb_MessageDef* msg_type =
         envoy_service_discovery_v3_DiscoveryResponse_getmsgdef(
             context.def_pool);
     char buf[10240];
     upb_TextEncode(reinterpret_cast<const upb_Message*>(response), msg_type,
                    nullptr, 0, buf, sizeof(buf));
-    gpr_log(GPR_DEBUG, "[xds_client %p] received response: %s", context.client,
-            buf);
+    VLOG(2) << "[xds_client " << context.client
+            << "] received response: " << buf;
   }
 }
 
@@ -351,16 +349,15 @@ namespace {
 void MaybeLogLrsRequest(
     const XdsApiContext& context,
     const envoy_service_load_stats_v3_LoadStatsRequest* request) {
-  if (GRPC_TRACE_FLAG_ENABLED(*context.tracer) &&
-      gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+  if (GRPC_TRACE_FLAG_ENABLED_OBJ(*context.tracer) && ABSL_VLOG_IS_ON(2)) {
     const upb_MessageDef* msg_type =
         envoy_service_load_stats_v3_LoadStatsRequest_getmsgdef(
             context.def_pool);
     char buf[10240];
     upb_TextEncode(reinterpret_cast<const upb_Message*>(request), msg_type,
                    nullptr, 0, buf, sizeof(buf));
-    gpr_log(GPR_DEBUG, "[xds_client %p] constructed LRS request: %s",
-            context.client, buf);
+    VLOG(2) << "[xds_client " << context.client
+            << "] constructed LRS request: " << buf;
   }
 }
 
@@ -513,16 +510,15 @@ namespace {
 void MaybeLogLrsResponse(
     const XdsApiContext& context,
     const envoy_service_load_stats_v3_LoadStatsResponse* response) {
-  if (GRPC_TRACE_FLAG_ENABLED(*context.tracer) &&
-      gpr_should_log(GPR_LOG_SEVERITY_DEBUG)) {
+  if (GRPC_TRACE_FLAG_ENABLED_OBJ(*context.tracer) && ABSL_VLOG_IS_ON(2)) {
     const upb_MessageDef* msg_type =
         envoy_service_load_stats_v3_LoadStatsResponse_getmsgdef(
             context.def_pool);
     char buf[10240];
     upb_TextEncode(reinterpret_cast<const upb_Message*>(response), msg_type,
                    nullptr, 0, buf, sizeof(buf));
-    gpr_log(GPR_DEBUG, "[xds_client %p] received LRS response: %s",
-            context.client, buf);
+    VLOG(2) << "[xds_client " << context.client
+            << "] received LRS response: " << buf;
   }
 }
 
