@@ -23,6 +23,7 @@
 #include <utility>
 #include <vector>
 
+#include "absl/log/log.h"
 #include "absl/strings/string_view.h"
 #include "absl/types/span.h"
 
@@ -36,10 +37,10 @@
 #include "src/core/lib/config/core_configuration.h"
 #include "src/core/lib/event_engine/channel_args_endpoint_config.h"
 #include "src/core/lib/event_engine/tcp_socket_utils.h"
-#include "src/core/lib/gpr/useful.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/slice/slice.h"
 #include "src/core/lib/slice/slice_buffer.h"
+#include "src/core/util/useful.h"
 #include "test/core/end2end/fuzzers/fuzzer_input.pb.h"
 #include "test/core/test_util/mock_endpoint.h"
 
@@ -383,23 +384,27 @@ std::vector<QueuedRead> MakeSchedule(
 
 Duration ScheduleReads(
     const fuzzer_input::NetworkInput& network_input,
-    grpc_endpoint* mock_endpoint,
+    std::shared_ptr<grpc_event_engine::experimental::MockEndpointController>
+        mock_endpoint_controller,
     grpc_event_engine::experimental::FuzzingEventEngine* event_engine) {
   int delay = 0;
   for (const auto& q : MakeSchedule(network_input)) {
     event_engine->RunAfterExactly(
         std::chrono::milliseconds(q.delay_ms),
-        [mock_endpoint, slices = q.slices.JoinIntoSlice()]() mutable {
+        [mock_endpoint_controller,
+         slices = q.slices.JoinIntoSlice()]() mutable {
           ExecCtx exec_ctx;
-          grpc_mock_endpoint_put_read(mock_endpoint, slices.TakeCSlice());
+          mock_endpoint_controller->TriggerReadEvent(
+              std::move(grpc_event_engine::experimental::internal::SliceCast<
+                        grpc_event_engine::experimental::Slice>(slices)));
         });
     delay = std::max(delay, q.delay_ms);
   }
-  event_engine->RunAfterExactly(
-      std::chrono::milliseconds(delay + 1), [mock_endpoint] {
-        ExecCtx exec_ctx;
-        grpc_mock_endpoint_finish_put_reads(mock_endpoint);
-      });
+  event_engine->RunAfterExactly(std::chrono::milliseconds(delay + 1),
+                                [mock_endpoint_controller] {
+                                  ExecCtx exec_ctx;
+                                  mock_endpoint_controller->NoMoreReads();
+                                });
   return Duration::Milliseconds(delay + 2);
 }
 
@@ -520,8 +525,7 @@ Duration ScheduleConnection(
                     endpoint) mutable {
               ExecCtx exec_ctx;
               if (!endpoint.ok()) {
-                gpr_log(GPR_ERROR, "Failed to connect: %s",
-                        endpoint.status().ToString().c_str());
+                LOG(ERROR) << "Failed to connect: " << endpoint.status();
                 return;
               }
               std::shared_ptr<EventEngine::Endpoint> ep =
