@@ -24,6 +24,7 @@
 #include <memory>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/strings/str_cat.h"
 #include "absl/strings/str_format.h"
@@ -32,7 +33,6 @@
 
 #include <grpc/event_engine/event_engine.h>
 #include <grpc/support/alloc.h>
-#include <grpc/support/log.h>
 
 #include "src/core/lib/address_utils/sockaddr_utils.h"
 #include "src/core/lib/gprpp/crash.h"
@@ -185,7 +185,7 @@ int WriteBytes(int sockfd, int& saved_errno, std::string write_bytes) {
       return ret;
     }
     write_bytes = write_bytes.substr(ret, std::string::npos);
-  } while (write_bytes.length() > 0);
+  } while (!write_bytes.empty());
   return original_write_length;
 }
 }  // namespace
@@ -255,7 +255,7 @@ bool PosixOracleEndpoint::Write(
 }
 
 void PosixOracleEndpoint::ProcessReadOperations() {
-  gpr_log(GPR_INFO, "Starting thread to process read ops ...");
+  LOG(INFO) << "Starting thread to process read ops ...";
   while (true) {
     read_op_signal_->WaitForNotification();
     read_op_signal_ = std::make_unique<grpc_core::Notification>();
@@ -273,11 +273,11 @@ void PosixOracleEndpoint::ProcessReadOperations() {
                                               grpc_core::StrError(saved_errno)))
                            : absl::OkStatus());
   }
-  gpr_log(GPR_INFO, "Shutting down read ops thread ...");
+  LOG(INFO) << "Shutting down read ops thread ...";
 }
 
 void PosixOracleEndpoint::ProcessWriteOperations() {
-  gpr_log(GPR_INFO, "Starting thread to process write ops ...");
+  LOG(INFO) << "Starting thread to process write ops ...";
   while (true) {
     write_op_signal_->WaitForNotification();
     write_op_signal_ = std::make_unique<grpc_core::Notification>();
@@ -293,7 +293,7 @@ void PosixOracleEndpoint::ProcessWriteOperations() {
                                         grpc_core::StrError(saved_errno)))
                      : absl::OkStatus());
   }
-  gpr_log(GPR_INFO, "Shutting down write ops thread ...");
+  LOG(INFO) << "Shutting down write ops thread ...";
 }
 
 PosixOracleListener::PosixOracleListener(
@@ -341,7 +341,7 @@ PosixOracleListener::~PosixOracleListener() {
 }
 
 void PosixOracleListener::HandleIncomingConnections() {
-  gpr_log(GPR_INFO, "Starting accept thread ...");
+  LOG(INFO) << "Starting accept thread ...";
   CHECK(!listener_fds_.empty());
   int nfds = listener_fds_.size();
   // Add one extra file descriptor to poll the pipe fd.
@@ -371,17 +371,16 @@ void PosixOracleListener::HandleIncomingConnections() {
       // pfds[i].fd has a readable event.
       int client_sock_fd = accept(pfds[i].fd, nullptr, nullptr);
       if (client_sock_fd < 0) {
-        gpr_log(GPR_ERROR,
-                "Error accepting new connection: %s. Ignoring connection "
-                "attempt ...",
-                grpc_core::StrError(errno).c_str());
+        LOG(ERROR) << "Error accepting new connection: "
+                   << grpc_core::StrError(errno)
+                   << ". Ignoring connection attempt ...";
         continue;
       }
       on_accept_(PosixOracleEndpoint::Create(client_sock_fd),
                  memory_allocator_factory_->CreateMemoryAllocator("test"));
     }
   }
-  gpr_log(GPR_INFO, "Shutting down accept thread ...");
+  LOG(INFO) << "Shutting down accept thread ...";
   gpr_free(pfds);
 }
 
