@@ -64,7 +64,6 @@ class BinderTransportTest : public ::testing::Test {
       gbs->~grpc_binder_stream();
       gpr_free(gbs);
     }
-    arena_->Destroy();
   }
 
   void PerformStreamOp(grpc_binder_stream* gbs,
@@ -81,7 +80,7 @@ class BinderTransportTest : public ::testing::Test {
     grpc_binder_stream* gbs = static_cast<grpc_binder_stream*>(
         gpr_malloc(transport_->filter_stack_transport()->SizeOfStream()));
     transport_->filter_stack_transport()->InitStream(
-        reinterpret_cast<grpc_stream*>(gbs), &ref_, nullptr, arena_);
+        reinterpret_cast<grpc_stream*>(gbs), &ref_, nullptr, arena_.get());
     stream_buffer_.push_back(gbs);
     return gbs;
   }
@@ -95,12 +94,8 @@ class BinderTransportTest : public ::testing::Test {
   static void TearDownTestSuite() { grpc_shutdown(); }
 
  protected:
-  grpc_core::MemoryAllocator memory_allocator_ =
-      grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
-                                     ->memory_quota()
-                                     ->CreateMemoryAllocator("test"));
-  grpc_core::Arena* arena_ =
-      grpc_core::Arena::Create(/* initial_size = */ 1, &memory_allocator_);
+  grpc_core::RefCountedPtr<grpc_core::Arena> arena_ =
+      grpc_core::SimpleArenaAllocator()->MakeArena();
   grpc_core::Transport* transport_;
   grpc_stream_refcount ref_;
   std::vector<grpc_binder_stream*> stream_buffer_;
@@ -234,12 +229,6 @@ struct MakeSendInitialMetadata {
   }
   ~MakeSendInitialMetadata() {}
 
-  grpc_core::MemoryAllocator memory_allocator =
-      grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
-                                     ->memory_quota()
-                                     ->CreateMemoryAllocator("test"));
-  grpc_core::ScopedArenaPtr arena =
-      grpc_core::MakeScopedArena(1024, &memory_allocator);
   grpc_metadata_batch grpc_initial_metadata;
 };
 
@@ -269,8 +258,6 @@ struct MakeSendTrailingMetadata {
       grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
                                      ->memory_quota()
                                      ->CreateMemoryAllocator("test"));
-  grpc_core::ScopedArenaPtr arena =
-      grpc_core::MakeScopedArena(1024, &memory_allocator);
   grpc_metadata_batch grpc_trailing_metadata;
 };
 
@@ -297,8 +284,6 @@ struct MakeRecvInitialMetadata {
       grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
                                      ->memory_quota()
                                      ->CreateMemoryAllocator("test"));
-  grpc_core::ScopedArenaPtr arena =
-      grpc_core::MakeScopedArena(1024, &memory_allocator);
   grpc_metadata_batch grpc_initial_metadata;
   grpc_core::Notification notification;
 };
@@ -345,8 +330,6 @@ struct MakeRecvTrailingMetadata {
       grpc_core::MemoryAllocator(grpc_core::ResourceQuota::Default()
                                      ->memory_quota()
                                      ->CreateMemoryAllocator("test"));
-  grpc_core::ScopedArenaPtr arena =
-      grpc_core::MakeScopedArena(1024, &memory_allocator);
   grpc_metadata_batch grpc_trailing_metadata;
   grpc_core::Notification notification;
 };
@@ -396,7 +379,7 @@ TEST_F(BinderTransportTest, PerformSendInitialMetadata) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
   const Metadata kInitialMetadata = kDefaultMetadata;
   MakeSendInitialMetadata send_initial_metadata(kInitialMetadata, "", &op);
@@ -416,7 +399,7 @@ TEST_F(BinderTransportTest, PerformSendInitialMetadataMethodRef) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
   const Metadata kInitialMetadata = kDefaultMetadata;
   const std::string kMethodRef = kDefaultMethodRef;
@@ -439,7 +422,7 @@ TEST_F(BinderTransportTest, PerformSendMessage) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   const std::string kMessage = kDefaultMessage;
@@ -461,7 +444,7 @@ TEST_F(BinderTransportTest, PerformSendTrailingMetadata) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
   // The wireformat guarantees that suffix metadata will always be empty.
   // TODO(waynetu): Check whether gRPC can internally add extra trailing
@@ -484,7 +467,7 @@ TEST_F(BinderTransportTest, PerformSendAll) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   const Metadata kInitialMetadata = kDefaultMetadata;
@@ -519,7 +502,7 @@ TEST_F(BinderTransportTest, PerformRecvInitialMetadata) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   MakeRecvInitialMetadata recv_initial_metadata(&op);
@@ -540,7 +523,7 @@ TEST_F(BinderTransportTest, PerformRecvInitialMetadataWithMethodRef) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   MakeRecvInitialMetadata recv_initial_metadata(&op);
@@ -562,7 +545,7 @@ TEST_F(BinderTransportTest, PerformRecvMessage) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   MakeRecvMessage recv_message(&op);
@@ -582,7 +565,7 @@ TEST_F(BinderTransportTest, PerformRecvTrailingMetadata) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   MakeRecvTrailingMetadata recv_trailing_metadata(&op);
@@ -604,7 +587,7 @@ TEST_F(BinderTransportTest, PerformRecvAll) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   MakeRecvInitialMetadata recv_initial_metadata(&op);
@@ -640,7 +623,7 @@ TEST_F(BinderTransportTest, PerformAllOps) {
   grpc_core::ExecCtx exec_ctx;
   grpc_binder_stream* gbs = InitNewBinderStream();
   grpc_transport_stream_op_batch op{};
-  grpc_transport_stream_op_batch_payload payload(nullptr);
+  grpc_transport_stream_op_batch_payload payload;
   op.payload = &payload;
 
   const Metadata kSendInitialMetadata = kDefaultMetadata;
@@ -723,13 +706,13 @@ TEST_F(BinderTransportTest, WireWriterRpcCallErrorPropagates) {
 
   const Metadata kInitialMetadata = {};
   grpc_transport_stream_op_batch op1{};
-  grpc_transport_stream_op_batch_payload payload1(nullptr);
+  grpc_transport_stream_op_batch_payload payload1;
   op1.payload = &payload1;
   MakeSendInitialMetadata send_initial_metadata1(kInitialMetadata, "", &op1);
   op1.on_complete = mock_on_complete1.GetGrpcClosure();
 
   grpc_transport_stream_op_batch op2{};
-  grpc_transport_stream_op_batch_payload payload2(nullptr);
+  grpc_transport_stream_op_batch_payload payload2;
   op2.payload = &payload2;
   MakeSendInitialMetadata send_initial_metadata2(kInitialMetadata, "", &op2);
   op2.on_complete = mock_on_complete2.GetGrpcClosure();
