@@ -20,6 +20,7 @@
 #include <string>
 #include <utility>
 
+#include "absl/log/log.h"
 #include "absl/status/status.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
@@ -28,16 +29,19 @@
 #include "absl/types/optional.h"
 
 #include <grpc/grpc.h>
-#include <grpc/support/log.h>
 
 #include "src/core/lib/event_engine/default_event_engine.h"
 #include "src/core/lib/gprpp/orphanable.h"
 #include "src/core/lib/gprpp/ref_counted_ptr.h"
 #include "src/core/xds/grpc/xds_bootstrap_grpc.h"
 #include "src/core/xds/grpc/xds_cluster.h"
+#include "src/core/xds/grpc/xds_cluster_parser.h"
 #include "src/core/xds/grpc/xds_endpoint.h"
+#include "src/core/xds/grpc/xds_endpoint_parser.h"
 #include "src/core/xds/grpc/xds_listener.h"
+#include "src/core/xds/grpc/xds_listener_parser.h"
 #include "src/core/xds/grpc/xds_route_config.h"
+#include "src/core/xds/grpc/xds_route_config_parser.h"
 #include "src/core/xds/xds_client/xds_bootstrap.h"
 #include "src/core/xds/xds_client/xds_client.h"
 #include "src/libfuzzer/libfuzzer_macro.h"
@@ -53,8 +57,7 @@ class Fuzzer {
   explicit Fuzzer(absl::string_view bootstrap_json) {
     auto bootstrap = GrpcXdsBootstrap::Create(bootstrap_json);
     if (!bootstrap.ok()) {
-      gpr_log(GPR_ERROR, "error creating bootstrap: %s",
-              bootstrap.status().ToString().c_str());
+      LOG(ERROR) << "error creating bootstrap: " << bootstrap.status();
       // Leave xds_client_ unset, so Act() will be a no-op.
       return;
     }
@@ -122,20 +125,18 @@ class Fuzzer {
                 [](const testing::XdsClientTestPeer::ResourceCountLabels&
                        labels,
                    uint64_t count) {
-                  gpr_log(GPR_INFO,
-                          "xds_authority=\"%s\", resource_type=\"%s\", "
-                          "cache_state=\"%s\" count=%" PRIu64,
-                          std::string(labels.xds_authority).c_str(),
-                          std::string(labels.resource_type).c_str(),
-                          std::string(labels.cache_state).c_str(), count);
+                  LOG(INFO) << "xds_authority=\"" << labels.xds_authority
+                            << "\", resource_type=\"" << labels.resource_type
+                            << "\", cache_state=\"" << labels.cache_state
+                            << "\" count=" << count;
                 });
         break;
       case xds_client_fuzzer::Action::kReportServerConnections:
         testing::XdsClientTestPeer(xds_client_.get())
             .TestReportServerConnections(
                 [](absl::string_view xds_server, bool connected) {
-                  gpr_log(GPR_INFO, "xds_server=\"%s\" connected=%d",
-                          std::string(xds_server).c_str(), connected);
+                  LOG(INFO) << "xds_server=\"" << xds_server
+                            << "\" connected=" << connected;
                 });
         break;
       case xds_client_fuzzer::Action::kTriggerConnectionFailure:
@@ -174,26 +175,24 @@ class Fuzzer {
         std::shared_ptr<const typename ResourceType::ResourceType> resource,
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
-      gpr_log(GPR_INFO, "==> OnResourceChanged(%s %s): %s",
-              std::string(ResourceType::Get()->type_url()).c_str(),
-              resource_name_.c_str(), resource->ToString().c_str());
+      LOG(INFO) << "==> OnResourceChanged(" << ResourceType::Get()->type_url()
+                << " " << resource_name_ << "): " << resource->ToString();
     }
 
     void OnError(
         absl::Status status,
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
-      gpr_log(GPR_INFO, "==> OnError(%s %s): %s",
-              std::string(ResourceType::Get()->type_url()).c_str(),
-              resource_name_.c_str(), status.ToString().c_str());
+      LOG(INFO) << "==> OnError(" << ResourceType::Get()->type_url() << " "
+                << resource_name_ << "): " << status;
     }
 
     void OnResourceDoesNotExist(
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
-      gpr_log(GPR_INFO, "==> OnResourceDoesNotExist(%s %s)",
-              std::string(ResourceType::Get()->type_url()).c_str(),
-              resource_name_.c_str());
+      LOG(INFO) << "==> OnResourceDoesNotExist("
+                << ResourceType::Get()->type_url() << " " << resource_name_
+                << ")";
     }
 
    private:
@@ -208,9 +207,9 @@ class Fuzzer {
   template <typename WatcherType>
   void StartWatch(std::map<std::string, std::set<WatcherType*>>* watchers,
                   std::string resource_name) {
-    gpr_log(GPR_INFO, "### StartWatch(%s %s)",
-            std::string(WatcherType::ResourceType::Get()->type_url()).c_str(),
-            resource_name.c_str());
+    LOG(INFO) << "### StartWatch("
+              << WatcherType::ResourceType::Get()->type_url() << " "
+              << resource_name << ")";
     auto watcher = MakeRefCounted<WatcherType>(resource_name);
     (*watchers)[resource_name].insert(watcher.get());
     WatcherType::ResourceType::Get()->StartWatch(
@@ -220,9 +219,9 @@ class Fuzzer {
   template <typename WatcherType>
   void StopWatch(std::map<std::string, std::set<WatcherType*>>* watchers,
                  std::string resource_name) {
-    gpr_log(GPR_INFO, "### StopWatch(%s %s)",
-            std::string(WatcherType::ResourceType::Get()->type_url()).c_str(),
-            resource_name.c_str());
+    LOG(INFO) << "### StopWatch("
+              << WatcherType::ResourceType::Get()->type_url() << " "
+              << resource_name << ")";
     auto& watchers_set = (*watchers)[resource_name];
     auto it = watchers_set.begin();
     if (it == watchers_set.end()) return;
@@ -252,8 +251,8 @@ class Fuzzer {
 
   void TriggerConnectionFailure(const std::string& authority,
                                 absl::Status status) {
-    gpr_log(GPR_INFO, "### TriggerConnectionFailure(%s): %s", authority.c_str(),
-            status.ToString().c_str());
+    LOG(INFO) << "### TriggerConnectionFailure(" << authority
+              << "): " << status;
     const auto* xds_server = GetServer(authority);
     if (xds_server == nullptr) return;
     transport_factory_->TriggerConnectionFailure(*xds_server,
@@ -290,14 +289,14 @@ class Fuzzer {
 
   void ReadMessageFromClient(const xds_client_fuzzer::StreamId& stream_id,
                              bool ok) {
-    gpr_log(GPR_INFO, "### ReadMessageFromClient(%s): %s",
-            StreamIdString(stream_id).c_str(), ok ? "true" : "false");
+    LOG(INFO) << "### ReadMessageFromClient(" << StreamIdString(stream_id)
+              << "): " << (ok ? "true" : "false");
     auto stream = GetStream(stream_id);
     if (stream == nullptr) return;
-    gpr_log(GPR_INFO, "    stream=%p", stream.get());
+    LOG(INFO) << "    stream=" << stream.get();
     auto message = stream->WaitForMessageFromClient(absl::ZeroDuration());
     if (message.has_value()) {
-      gpr_log(GPR_INFO, "    completing send_message");
+      LOG(INFO) << "    completing send_message";
       stream->CompleteSendMessageFromClient(ok);
     }
   }
@@ -305,21 +304,20 @@ class Fuzzer {
   void SendMessageToClient(
       const xds_client_fuzzer::StreamId& stream_id,
       const envoy::service::discovery::v3::DiscoveryResponse& response) {
-    gpr_log(GPR_INFO, "### SendMessageToClient(%s)",
-            StreamIdString(stream_id).c_str());
+    LOG(INFO) << "### SendMessageToClient(" << StreamIdString(stream_id) << ")";
     auto stream = GetStream(stream_id);
     if (stream == nullptr) return;
-    gpr_log(GPR_INFO, "    stream=%p", stream.get());
+    LOG(INFO) << "    stream=" << stream.get();
     stream->SendMessageToClient(response.SerializeAsString());
   }
 
   void SendStatusToClient(const xds_client_fuzzer::StreamId& stream_id,
                           absl::Status status) {
-    gpr_log(GPR_INFO, "### SendStatusToClient(%s): %s",
-            StreamIdString(stream_id).c_str(), status.ToString().c_str());
+    LOG(INFO) << "### SendStatusToClient(" << StreamIdString(stream_id)
+              << "): " << status;
     auto stream = GetStream(stream_id);
     if (stream == nullptr) return;
-    gpr_log(GPR_INFO, "    stream=%p", stream.get());
+    LOG(INFO) << "    stream=" << stream.get();
     stream->MaybeSendStatusToClient(std::move(status));
   }
 
