@@ -19,6 +19,7 @@
 #include <gtest/gtest.h>
 
 #include "absl/log/check.h"
+#include "absl/log/log.h"
 #include "absl/status/statusor.h"
 #include "absl/strings/str_cat.h"
 
@@ -46,19 +47,27 @@ using ::envoy::extensions::clusters::aggregate::v3::ClusterConfig;
 
 class ClusterTypeTest : public XdsEnd2endTest {
  protected:
-  void SetUp() override {
-    logical_dns_cluster_resolver_response_generator_ =
-        grpc_core::MakeRefCounted<grpc_core::FakeResolverResponseGenerator>();
-    InitClient();
+  ClusterTypeTest()
+      : logical_dns_cluster_resolver_response_generator_(
+            grpc_core::MakeRefCounted<
+                grpc_core::FakeResolverResponseGenerator>()) {}
+
+  // Subclasses must call this to initialize.
+  void LogicalDnsInitClient(
+      absl::optional<XdsBootstrapBuilder> builder = absl::nullopt,
+      std::shared_ptr<ChannelCredentials> credentials = nullptr) {
     ChannelArguments args;
     args.SetPointerWithVtable(
         GRPC_ARG_XDS_LOGICAL_DNS_CLUSTER_FAKE_RESOLVER_RESPONSE_GENERATOR,
         logical_dns_cluster_resolver_response_generator_.get(),
         &grpc_core::FakeResolverResponseGenerator::kChannelArgPointerVtable);
-    ResetStub(/*failover_timeout_ms=*/0, &args);
+    InitClient(builder, /*lb_expected_authority=*/"",
+               /*xds_resource_does_not_exist_timeout_ms=*/0,
+               /*balancer_authority_override=*/"", /*args=*/&args,
+               std::move(credentials));
   }
 
-  grpc_core::EndpointAddressesList CreateAddressListFromPortList(
+  static grpc_core::EndpointAddressesList CreateAddressListFromPortList(
       const std::vector<int>& ports) {
     grpc_core::EndpointAddressesList addresses;
     for (int port : ports) {
@@ -80,12 +89,16 @@ class ClusterTypeTest : public XdsEnd2endTest {
 // LOGICAL_DNS cluster tests
 //
 
-using LogicalDNSClusterTest = ClusterTypeTest;
+class LogicalDNSClusterTest : public ClusterTypeTest {
+ protected:
+  void SetUp() override {}  // Individual tests call LogicalDnsInitClient().
+};
 
 INSTANTIATE_TEST_SUITE_P(XdsTest, LogicalDNSClusterTest,
                          ::testing::Values(XdsTestType()), &XdsTestType::Name);
 
 TEST_P(LogicalDNSClusterTest, Basic) {
+  LogicalDnsInitClient();
   CreateAndStartBackends(1);
   // Create Logical DNS Cluster
   auto cluster = default_cluster_;
@@ -111,6 +124,226 @@ TEST_P(LogicalDNSClusterTest, Basic) {
   CheckRpcSendOk(DEBUG_LOCATION);
 }
 
+TEST_P(LogicalDNSClusterTest, FailedBackendConnectionCausesReresolution) {
+  LogicalDnsInitClient();
+  CreateAndStartBackends(2);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kServerName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Set Logical DNS result to backend 0.
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts(0, 1));
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // RPCs should succeed.
+  CheckRpcSendOk(DEBUG_LOCATION);
+  // Now shut down backend 0.
+  ShutdownBackend(0);
+  // Wait for logical DNS resolver to see a re-resolution request.
+  // Then return a DNS result pointing to backend 1.
+  {
+    grpc_core::ExecCtx exec_ctx;
+    ASSERT_TRUE(logical_dns_cluster_resolver_response_generator_
+                    ->WaitForReresolutionRequest(absl::Seconds(10) *
+                                                 grpc_test_slowdown_factor()));
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts(1, 2));
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Wait for traffic to switch to backend 1.
+  // RPCs may fail until the client sees the resolver result.
+  WaitForBackend(DEBUG_LOCATION, 1, [](const RpcResult& result) {
+    if (!result.status.ok()) {
+      EXPECT_EQ(StatusCode::UNAVAILABLE, result.status.error_code());
+      EXPECT_THAT(result.status.error_message(),
+                  MakeConnectionFailureRegex(
+                      "connections to all backends failing; last error: "));
+    }
+  });
+}
+
+TEST_P(LogicalDNSClusterTest, AutoHostRewrite) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_AUTHORITY_REWRITE");
+  constexpr char kDnsName[] = "dns.example.com";
+  // Set auto_host_rewrite in the RouteConfig.
+  RouteConfiguration new_route_config = default_route_config_;
+  new_route_config.mutable_virtual_hosts(0)
+      ->mutable_routes(0)
+      ->mutable_route()
+      ->mutable_auto_host_rewrite()
+      ->set_value(true);
+  SetRouteConfiguration(balancer_.get(), new_route_config);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  // Note: We use InsecureCreds, since FakeCreds are too picky about
+  // what authority gets sent.
+  LogicalDnsInitClient(MakeBootstrapBuilder().SetTrustedXdsServer(),
+                       InsecureChannelCredentials());
+  CreateAndStartBackends(1, /*xds_enabled=*/false, InsecureServerCredentials());
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), absl::StrCat(kDnsName, ":443"));
+}
+
+TEST_P(LogicalDNSClusterTest, NoAuthorityRewriteWithoutEnvVar) {
+  constexpr char kDnsName[] = "dns.example.com";
+  // Set auto_host_rewrite in the RouteConfig.
+  RouteConfiguration new_route_config = default_route_config_;
+  new_route_config.mutable_virtual_hosts(0)
+      ->mutable_routes(0)
+      ->mutable_route()
+      ->mutable_auto_host_rewrite()
+      ->set_value(true);
+  SetRouteConfiguration(balancer_.get(), new_route_config);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  LogicalDnsInitClient(MakeBootstrapBuilder().SetTrustedXdsServer());
+  CreateAndStartBackends(1);
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), kServerName);
+}
+
+TEST_P(LogicalDNSClusterTest, NoAuthorityRewriteIfServerNotTrustedInBootstrap) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_AUTHORITY_REWRITE");
+  constexpr char kDnsName[] = "dns.example.com";
+  // Set auto_host_rewrite in the RouteConfig.
+  RouteConfiguration new_route_config = default_route_config_;
+  new_route_config.mutable_virtual_hosts(0)
+      ->mutable_routes(0)
+      ->mutable_route()
+      ->mutable_auto_host_rewrite()
+      ->set_value(true);
+  SetRouteConfiguration(balancer_.get(), new_route_config);
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  LogicalDnsInitClient();
+  CreateAndStartBackends(1);
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), kServerName);
+}
+
+TEST_P(LogicalDNSClusterTest, NoAuthorityRewriteIfNotEnabledInRoute) {
+  grpc_core::testing::ScopedExperimentalEnvVar env(
+      "GRPC_EXPERIMENTAL_XDS_AUTHORITY_REWRITE");
+  constexpr char kDnsName[] = "dns.example.com";
+  // Create Logical DNS Cluster
+  auto cluster = default_cluster_;
+  cluster.set_type(Cluster::LOGICAL_DNS);
+  auto* address = cluster.mutable_load_assignment()
+                      ->add_endpoints()
+                      ->add_lb_endpoints()
+                      ->mutable_endpoint()
+                      ->mutable_address()
+                      ->mutable_socket_address();
+  address->set_address(kDnsName);
+  address->set_port_value(443);
+  balancer_->ads_service()->SetCdsResource(cluster);
+  // Create client and server.
+  LogicalDnsInitClient(MakeBootstrapBuilder().SetTrustedXdsServer());
+  CreateAndStartBackends(1);
+  // Set Logical DNS result
+  {
+    grpc_core::ExecCtx exec_ctx;
+    grpc_core::Resolver::Result result;
+    result.addresses = CreateAddressListFromPortList(GetBackendPorts());
+    logical_dns_cluster_resolver_response_generator_->SetResponseSynchronously(
+        std::move(result));
+  }
+  // Send RPC and verify the authority seen by the server.
+  EchoResponse response;
+  Status status = SendRpc(
+      RpcOptions().set_echo_host_from_authority_header(true), &response);
+  EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
+                           << " message=" << status.error_message();
+  EXPECT_EQ(response.param().host(), kServerName);
+}
+
 //
 // aggregate cluster tests
 //
@@ -118,7 +351,10 @@ TEST_P(LogicalDNSClusterTest, Basic) {
 // TODO(roth): Add tests showing that load reporting is enabled on a
 // per-underlying-cluster basis.
 
-using AggregateClusterTest = ClusterTypeTest;
+class AggregateClusterTest : public ClusterTypeTest {
+ protected:
+  void SetUp() override { LogicalDnsInitClient(); }
+};
 
 INSTANTIATE_TEST_SUITE_P(XdsTest, AggregateClusterTest,
                          ::testing::Values(XdsTestType()), &XdsTestType::Name);
@@ -367,9 +603,9 @@ TEST_P(AggregateClusterTest, FallBackWithConnectivityChurn) {
   // Meanwhile, the channel will also start a second attempt for backend
   // 0, which we have NOT held, so it will complete normally, and the
   // RPC will finish on backend 0.
-  gpr_log(GPR_INFO, "=== WAITING FOR RPC TO FINISH === ");
+  LOG(INFO) << "=== WAITING FOR RPC TO FINISH === ";
   Status status = rpc.GetStatus();
-  gpr_log(GPR_INFO, "=== RPC FINISHED === ");
+  LOG(INFO) << "=== RPC FINISHED === ";
   EXPECT_TRUE(status.ok()) << "code=" << status.error_code()
                            << " message=" << status.error_message();
   EXPECT_EQ(1UL, backends_[0]->backend_service()->request_count());
